/****************************************************************************
 * @file: MCM.h
 * @author: Young Leon
 * @version: V1.0
 * @data: 2022/10/18
 * @brief: Motor control meachine header file
 * @note:
 * @Copyright (C) 2022 Novosense All rights reserved.
 */
#ifndef __MCM_H__
#define __MCM_H__
/*********************************** INCLUDE ***********************************/

#include "main.h"

/*********************************** DEFINITION ***********************************/

/**
 * @brief transfer CAPCOM counter into speed in RPM
 *
 */
#define MCM_SPEED_FACTOR (FBK_CAPCOM_CLK_HZ * 60 / MOTOR_PP)

enum MCM_StateMeachine_t {
    MCM_OFF = 0,
    MCM_START_CCW,
    MCM_CCW,
    MCM_START_CCW_STEP,
    MCM_CCW_STEP,
    MCM_START_CW,
    MCM_CW,
    MCM_START_CW_STEP,
    MCM_CW_STEP,
    MCM_CW_CCW_STEP_IDLE,
    MCM_OFF_STEP,
    MCM_DIRECT_BRAKE,
    MCM_CTRL_BRAKE,
};

/**
 * @brief incremental pid structure
 *
 */
typedef struct {
    IQ15_t curErr;
    IQ15_t lastErr;
    IQ15_t preErr;
    IQ15_t kP;
    IQ15_t kI;
    IQ15_t ouput;
    IQ15_t curTarget;
    IQ15_t refTarget;
    IQ15_t input;
    IQ15_t dTarget;
    IQ15_t targetAdd;
    int16_t rampOrder;
} IncPid_t;

/*********************************** INTERFACE **********************************/
extern uint16_t g_MCMPwmWidth;

void MCM_SetMotorOff(void);
void MCM_StartCWRotate(void);
void MCM_StartCCWRotate(void);
void MCM_StartCWStep(void);
void MCM_StartCCWStep(void);
void MCM_GoOneCWStep(void);
void MCM_GoOneCCWStep(void);
void MCM_DirectBrake(void);
void MCM_ControlBrake(void);
uint8_t MCM_GetStatus(void);
void MCM_SetTargetRPM(uint16_t tarRPM);
uint16_t MCM_GetTargetRPM(void);
uint16_t MCM_GetCurrentRPM(void);

#endif //__MCM_H__
