/****************************************************************************
 * @file: MOT.h
 * @author: Young Leon
 * @version: V1.0
 * @data: 2022/10/18
 * @brief: Motor header file
 * @note:
 * @Copyright (C) 2022 Novosense All rights reserved.
 */

#ifndef __MOT_H__
#define __MOT_H__
/*********************************** INCLUDE ***********************************/
#include "main.h"

/*********************************** DEFINITION ***********************************/
/**
 * Motor sector definition.
 * Value of the hall sensor for each sector:
  | Sector Name  | Hall Value |	 HA |  HB |  HC |
  |:------------:|:----------:|:---:|:---:|:---:|
  | MOT_SECTOR_0 |       5    |  1  |  0  |  1  |
  | MOT_SECTOR_1 |       4    |  1  |  0  |  0  |
  | MOT_SECTOR_2 |       6    |  1  |  1  |  0  |
  | MOT_SECTOR_3 |       2    |  0  |  1  |  0  |
  | MOT_SECTOR_4 |       3    |  0  |  1  |  1  |
  | MOT_SECTOR_5 |       1    |  0  |  0  |  1  |
*/

/**
 * Motor sector definition. for Nidec
 * Value of the hall sensor for each sector:
  | Sector Name  | Hall Value |	 HC |  HB |  HA |
  |:------------:|:----------:|:---:|:---:|:---:|
  | MOT_SECTOR_0 |       1    |  0  |  0  |  1  |
  | MOT_SECTOR_1 |       3    |  0  |  1  |  1  |
  | MOT_SECTOR_2 |       2    |  0  |  1  |  0  |
  | MOT_SECTOR_3 |       6    |  1  |  1  |  0  |
  | MOT_SECTOR_4 |       4    |  1  |  0  |  0  |
  | MOT_SECTOR_5 |       5    |  1  |  0  |  1  |
*/

/** Hall sensor pattern for sector 0 */
#define MOT_SECTOR_0                  (6) //(1)//(0x05U)
/** Hall sensor pattern for sector 1 */
#define MOT_SECTOR_1                  (4) //(3)//(0x04U)
/** Hall sensor pattern for sector 2 */
#define MOT_SECTOR_2                  (5) //(2)//(0x06U)
/** Hall sensor pattern for sector 3 */
#define MOT_SECTOR_3                  (1) //(6)//(0x02U)
/** Hall sensor pattern for sector 4 */
#define MOT_SECTOR_4                  (3) //(4)//(0x03U)
/** Hall sensor pattern for sector 5 */
#define MOT_SECTOR_5                  (2)//(5)//(0x01U)

/** Motor constant definiton, direction = CCW */
#define MOT_CCW                       (1)
/** Motor constant definiton, direction = CW */
#define MOT_CW                        (-1)
/** Motor constant definiton, direction = undefined */
#define MOT_UNDEF_DIR                 (0)
/** Motor constant definiton for g_MOTCommutationMode */
#define MOT_CCW_COMMUTATION_ENABLED   (1)
/** Motor constant definiton for g_MOTCommutationMode */
#define MOT_CW_COMMUTATION_ENABLED    (-1)
/** Motor constant definiton for g_MOTCommutationMode */
#define MOT_COMMUTATION_DISABLED      (0)
/** Motor constant definiton for fixed EPWM duty cycles */
#define MOT_COMPARE_VALUE_OFF         MOT_EPWM_PERIOD_COMPARE
/** Motor constant definiton for fixed EPWM duty cycles */
#define MOT_COMPARE_VALUE_ON          (0)
/** Motor constant definiton to convert set value to EPWM duty cycle reload value */
#define MOT_SetCompareValue(PwmWidth) ((PwmWidth))

/*********************************** INTERFACE ***********************************/

extern int8_t g_MOTActualRotorDir;
extern uint8_t g_MOTFaultStatus;
extern uint8_t g_MOTCcwSequenceTable[8];
extern uint8_t g_MOTCwSequenceTable[8];

/*********************************** PUBLICK FUNCTION ****************************/
void MOT_Init(void);
void MOT_SwitchMotorOff(void);
void MOT_StartCWRotation(void);
void MOT_StartCCWRotation(void);
void MOT_ChangeEpwmWidth(void);
void MOT_BrakeMotor(void);
void MOT_CommutatePhases(void);

void MOT_UpdateActualRotorDirection(void);
void MOT_StartCWStep(void);
void MOT_StartCCWStep(void);
void MOT_GoOneStepCW(void);
void MOT_GoOneStepCCW(void);
void MOT_LeaveMotorStep(void);


#endif // __MOT_H__
