/****************************************************************************
 * @file: Qmath.h
 * @author: Young Leon
 * @version: V1.0
 * @data: 2022/10/18
 * @brief: Q-format math lib
 * @note:
 * @Copyright (C) 2022 Novosense All rights reserved.
 */

#ifndef __QMATH_H__
#define __QMATH_H__
#include "stdint.h"

/* define unsigned Q-format type*/
typedef uint16_t UQ16_t;
typedef uint16_t UQ15_t;
typedef uint16_t UQ14_t;
typedef uint16_t UQ13_t;
typedef uint16_t UQ12_t;
typedef uint16_t UQ11_t;
typedef uint16_t UQ10_t;
typedef uint16_t UQ9_t;
typedef uint16_t UQ8_t;
typedef uint16_t UQ7_t;
typedef uint16_t UQ6_t;
typedef uint16_t UQ5_t;
typedef uint16_t UQ4_t;
typedef uint16_t UQ3_t;
typedef uint16_t UQ2_t;
typedef uint16_t UQ1_t;

/* define signed Q-format type*/
typedef int16_t IQ15_t;
typedef int16_t IQ14_t;
typedef int16_t IQ13_t;
typedef int16_t IQ12_t;
typedef int16_t IQ11_t;
typedef int16_t IQ10_t;
typedef int16_t IQ9_t;
typedef int16_t IQ8_t;
typedef int16_t IQ7_t;
typedef int16_t IQ6_t;
typedef int16_t IQ5_t;
typedef int16_t IQ4_t;
typedef int16_t IQ3_t;
typedef int16_t IQ2_t;
typedef int16_t IQ1_t;

/* define unsigned Q-format bit */
#define UQ16                      16
#define UQ15                      15
#define UQ14                      14
#define UQ13                      13
#define UQ12                      12
#define UQ11                      11
#define UQ10                      10
#define UQ9                       9
#define UQ8                       8
#define UQ7                       7
#define UQ6                       6
#define UQ5                       5
#define UQ4                       4
#define UQ3                       3
#define UQ2                       2
#define UQ1                       1

/* define signed Q-formant bit */
#define IQ15                      15
#define IQ14                      14
#define IQ13                      13
#define IQ12                      12
#define IQ11                      11
#define IQ10                      10
#define IQ9                       9
#define IQ8                       8
#define IQ7                       7
#define IQ6                       6
#define IQ5                       5
#define IQ4                       4
#define IQ3                       3
#define IQ2                       2
#define IQ1                       1

/* transfer float into unsigned Q-format */
#define _UQ16(a)                  ((uint16_t)(a * (1 << UQ16)))
#define _UQ15(a)                  ((uint16_t)(a * (1 << UQ15)))
#define _UQ14(a)                  ((uint16_t)(a * (1 << UQ14)))
#define _UQ13(a)                  ((uint16_t)(a * (1 << UQ13)))
#define _UQ12(a)                  ((uint16_t)(a * (1 << UQ12)))
#define _UQ11(a)                  ((uint16_t)(a * (1 << UQ11)))
#define _UQ10(a)                  ((uint16_t)(a * (1 << UQ10)))
#define _UQ9(a)                   ((uint16_t)(a * (1 << UQ9)))
#define _UQ8(a)                   ((uint16_t)(a * (1 << UQ8)))
#define _UQ7(a)                   ((uint16_t)(a * (1 << UQ7)))
#define _UQ6(a)                   ((uint16_t)(a * (1 << UQ6)))
#define _UQ5(a)                   ((uint16_t)(a * (1 << UQ5)))
#define _UQ4(a)                   ((uint16_t)(a * (1 << UQ4)))
#define _UQ3(a)                   ((uint16_t)(a * (1 << UQ3)))
#define _UQ2(a)                   ((uint16_t)(a * (1 << UQ2)))
#define _UQ1(a)                   ((uint16_t)(a * (1 << UQ1)))

/* transfer unsigned Q-format into float*/
#define _fUQ16(a)                 ((float)((float)(a) / (float)(1 << UQ16)))
#define _fUQ15(a)                 ((float)((float)(a) / (float)(1 << UQ15)))
#define _fUQ14(a)                 ((float)((float)(a) / (float)(1 << UQ14)))
#define _fUQ13(a)                 ((float)((float)(a) / (float)(1 << UQ13)))
#define _fUQ12(a)                 ((float)((float)(a) / (float)(1 << UQ12)))
#define _fUQ11(a)                 ((float)((float)(a) / (float)(1 << UQ11)))
#define _fUQ10(a)                 ((float)((float)(a) / (float)(1 << UQ10)))
#define _fUQ9(a)                  ((float)((float)(a) / (float)(1 << UQ9)))
#define _fUQ8(a)                  ((float)((float)(a) / (float)(1 << UQ8)))
#define _fUQ7(a)                  ((float)((float)(a) / (float)(1 << UQ7)))
#define _fUQ6(a)                  ((float)((float)(a) / (float)(1 << UQ6)))
#define _fUQ5(a)                  ((float)((float)(a) / (float)(1 << UQ5)))
#define _fUQ4(a)                  ((float)((float)(a) / (float)(1 << UQ4)))
#define _fUQ3(a)                  ((float)((float)(a) / (float)(1 << UQ3)))
#define _fUQ2(a)                  ((float)((float)(a) / (float)(1 << UQ2)))
#define _fUQ1(a)                  ((float)((float)(a) / (float)(1 << UQ1)))

/* transfer float into signed Q-format */
#define _IQ15(a)                  ((int16_t)(a * (1 << IQ15)))
#define _IQ14(a)                  ((int16_t)(a * (1 << IQ14)))
#define _IQ13(a)                  ((int16_t)(a * (1 << IQ13)))
#define _IQ12(a)                  ((int16_t)(a * (1 << IQ12)))
#define _IQ11(a)                  ((int16_t)(a * (1 << IQ11)))
#define _IQ10(a)                  ((int16_t)(a * (1 << IQ10)))
#define _IQ9(a)                   ((int16_t)(a * (1 << IQ9)))
#define _IQ8(a)                   ((int16_t)(a * (1 << IQ8)))
#define _IQ7(a)                   ((int16_t)(a * (1 << IQ7)))
#define _IQ6(a)                   ((int16_t)(a * (1 << IQ6)))
#define _IQ5(a)                   ((int16_t)(a * (1 << IQ5)))
#define _IQ4(a)                   ((int16_t)(a * (1 << IQ4)))
#define _IQ3(a)                   ((int16_t)(a * (1 << IQ3)))
#define _IQ2(a)                   ((int16_t)(a * (1 << IQ2)))
#define _IQ1(a)                   ((int16_t)(a * (1 << IQ1)))

/* transfer signed Q-format into float*/
#define _fIQ15(a)                 ((float)((float)(a) / (float)(1 << IQ15)))
#define _fIQ14(a)                 ((float)((float)(a) / (float)(1 << IQ14)))
#define _fIQ13(a)                 ((float)((float)(a) / (float)(1 << IQ13)))
#define _fIQ12(a)                 ((float)((float)(a) / (float)(1 << IQ12)))
#define _fIQ11(a)                 ((float)((float)(a) / (float)(1 << IQ11)))
#define _fIQ10(a)                 ((float)((float)(a) / (float)(1 << IQ10)))
#define _fIQ9(a)                  ((float)((float)(a) / (float)(1 << IQ9)))
#define _fIQ8(a)                  ((float)((float)(a) / (float)(1 << IQ8)))
#define _fIQ7(a)                  ((float)((float)(a) / (float)(1 << IQ7)))
#define _fIQ6(a)                  ((float)((float)(a) / (float)(1 << IQ6)))
#define _fIQ5(a)                  ((float)((float)(a) / (float)(1 << IQ5)))
#define _fIQ4(a)                  ((float)((float)(a) / (float)(1 << IQ4)))
#define _fIQ3(a)                  ((float)((float)(a) / (float)(1 << IQ3)))
#define _fIQ2(a)                  ((float)((float)(a) / (float)(1 << IQ2)))
#define _fIQ1(a)                  ((float)((float)(a) / (float)(1 << IQ1)))

/* signed Q-format arithmetic */
#define QMATH_IQADD16(a, b)       __ssat(((a) + (b)), 16)
#define QMATH_IQSUB16_SAT(a, b)   __ssat((((a) - (b))), 16)
#define QMATH_IQSUB16(a, b)       (((a) - (b)))
#define QMATH_IQMUL16(a, b, qnum) __ssat((((int32_t)(a)) * ((int32_t)(b))) >> (qnum), 16)
#define QMATH_IQDIV16(a, b)       ((int32_t)(a) / (b))
#define QMATH_IQDIV16_RND(a, b, qnum)                                                                                  \
    (((a) >= 0 && (b) >= 0) || ((a) < 0 && (b) < 0)) ? (int16_t)((((int32_t)(a) << (qnum)) + (b) / 2) / (b))           \
                                                     : (int16_t)((((int32_t)(a) << (qnum)) - (b) / 2) / (b))

/* unsigned Q-format arithmetic */
#define QMATH_UQADD16(a, b)        __usat(((a) + (b)), 16)
#define QMATH_UQSUB16_SAT(a, b)    __usat((((a) - (b))), 16)
#define QMATH_UQSUB16(a, b)        (((a) - (b)))
#define QMATH_UQMUL16(a, b, uqnum) __usat((((uint32_t)(a)) * ((uint32_t)(b))) >> (uqnum), 16)
#define QMATH_UQDIV16(a, b, uqnum) (((uint32_t)(a) << (uqnum)) / (b))

#endif //__QMATH_H__
