/****************************************************************************
 * @file: sensor bldc config.h
 * @author: Young Leon
 * @version: V1.0
 * @data: 2022/04/22
 * @brief: config file for sensor BLDC controller
 * @note:
 * @Copyright (C) 2022 Novosense All rights reserved.
 */

#ifndef __CONFIG_H__
#define __CONFIG_H__

// motor pole-pairs
#define MOTOR_PP                (1)

/******************** controller definition ********************/
/* speed controller gain p */
#define MCM_GAINP               (0.01)
/* speed controller gain I */
#define MCM_GAINI               (0.015)
/* speed controller soft start order */
// the larger value, the slower ramp rate.
#define MCM_RAMP_ORDER          (16)

/* using speed control or not*/
#define MCM_SPEED_CONTROL       (1)

/* maximum speed in RPM*/
#define MCM_MAX_RPM             (8000)

/* the period of speed controller */
#define MCM_CTRL_PERIOD         (20) // control period = MCM_CTRL_PERIOD / MOT_EPWM_FREQUENCY_HZ (s) 20:1ms

/* ramp control period*/
#define MCM_RAMP_PERIOD         (50) // ramp period = MCM_RAMP_PERIOD * MCM_CTRL_PERIOD

/* sensorless definition*/
/* Align duration */
#define MCM_ALIGN_TIME          (20000) // Align duration: MCM_ALIGN_TIME / FBK_CAPCOM_CLK_HZ (s) 0.1s
										// Align duty using MCM_STARTUP_DUTY

/* Startup initial speed */
#define MCM_STARTUP_INITIAL_SPEED (200) // 50 RPM start up
// end speed will be MCM_STARTUP_INITIAL_SPEED/(MCM_STARTUP_ACC_RATIO^16)
#define MCM_STARTUP_ACC_RATIO   (0.9) // t[n] = t[n-1] * MCM_STARTUP_ACC_RATIO

/* Startup and Align duty */
#define MCM_STARTUP_DUTY        (100) // startup duty and align duty

#define MCM_STARTUP_DUTY_ACC    (1.05) // commutating duty: D[n] = D[n-1] * MCM_STARTUP_DUTY_ACC

/********************CAPCOM config defintion**********************/
/* CAPCOM clock prescaler */
#define FBK_CAPCOM_CLK_PS       (159)

/* CAPCOM frequency */
#define FBK_CAPCOM_CLK_HZ       (SYS_CLK / (FBK_CAPCOM_CLK_PS + 1))

// CAPCOM filter time: 0.264us*N
#define FBK_CAPCOM_FILTER_TIME  (0)

/* BEMFC blanking time
	Setting | time [us]
	--------|----------
	0		| 0
	1		| 1.4 to 1.6
	2		| 3.0 to 3.2
	3		| 4.6 to 4.8
	4		| 6.2 to 6.4
	5		| 7.8 to 8.0
	6		| 9.4 to 9.6
	...	| ...
	14 	| 22.2 to 22.4
	15	| 23.8 to 24.0
*/
#define FBK_BEMFC_BLANKING_TIME (2)

/* BEMFC deglitch time
 Setting  | time [us]
 -------- | -------------
 0  			| 0 (bypassed)
 1  			| 0.6 to 0.8
 2  			| 1.0 to 1.2
 3  			| 3.0 to 3.2
*/
#define FBK_BEMFC_DEGLITCH_FILTER_TIME		(3)

/*********** Hall capture interrupt and CAPCOM overflow ***********/
#define CAPCOM0_IRQ_PRIO        (0)

/** Hall capture interrupt */
#define CAPCOM1_IRQ_PRIO        (0)

/** Hall capture interrupt */
#define CAPCOM2_IRQ_PRIO        (0)

/******************** EPWM config definition *********** *********/
/** EPWM alignment mode.0: center 1: edge */
#define MOT_EPWM_PAM            (0)

/** EPWM input clock fpwm=fsys/2^N*/
#define MOT_EPWM_FSEL           (0)

/** EPWM period compare setting. */
#define MOT_EPWM_PERIOD_COMPARE (800) // period = fepwm / (MOT_EPWM_PERIOD_COMPARE * 2)

/** EPWM maximum output. percent %.*/
#define MOT_EPWM_MAX_OUTPUT     (95)

/** EPWM period(HZ). 32Mhz/(2*800). 20kHZ*/
#define MOT_EPWM_FREQUENCY_HZ   (SYS_CLK / (2 * MOT_EPWM_PERIOD_COMPARE))

/** EPWM switch on slew rate
 * 0: normal 1:reserved 2:fast 3: very fast
 */
#define MOT_EPWM_FSON           (0)

/** EPWM switch off slew rate
 * 0:normal 1:reserved 2:fast 3: very fast
 */
#define MOT_EPWM_FSOFF          (0)

/** EPWM over-current filter time(on 32Mhz).
 * 0:2.4-2.8us 1:4.8-5.6us
 * 2:9.6-11.2us 3:19.2-22.4us
 */
#define MOT_EPWM_OCFT           (3)

/** MOUT freewheeling and decay setting.
  Setting | FRWHL and Decay
    --------|-----------------
    0       | passive/asynchronous low-side freewheeling, slow decay (high-side PWM)
    1       | passive/asynchronous high-side freewheeling, slow decay (low-side PWM) --> NOT RECOMMENDED
    2       | active/synchronous low-side freewheeling, slow decay (complementary PWM, active high)
    3       | active/synchronous high-side freewheeling, slow decay (complementary PWM, active low)
    4       | passive/asynchronous freewheeling with fast decay (high- and low-side PWM, supply feedback) --> NOT
  RECOMMENDED
*/
#define MOT_FRWHL               (0)

/* EPWM interrupt priority*/
#define MOT_EPWM1_IRQ_PRIO      (2)

#define MOT_EPWM0_IRQ_PRIO      (1)

#define MOT_DEMAGNETIZATION_TIME (5)

#endif //__CONFIG_H__
