/****************************************************************************
 * @file    : system.c
 * @author  : novosense
 * @version : V1.1.0
 * @date    : 03-05-2018
 * @brief   : CMSIS Cortex-M3 Device Peripheral Access Layer System Source File.
 *            This file contains the system clock configuration for Chip devices,
 *            and is generated by the clock configuration tool.
 * @note
 * Copyright (C) 2019 novosense All rights reserved.
 ***************************************************************************/

 
// -------------------------------------------------------------------------//
// Includes
// -------------------------------------------------------------------------//
#include "include.h"

/*----------------------------------------------------------------------------
  Define clocks
 *----------------------------------------------------------------------------*/
#define __XTAL            (32000000UL)    /* Oscillator frequency             */

#define __SYSTEM_CLOCK    (__XTAL)

extern uint32_t __Vectors;
/*----------------------------------------------------------------------------
  Clock Variable definitions
 *----------------------------------------------------------------------------*/
uint32_t SystemCoreClock = __SYSTEM_CLOCK;/*!< System Clock Frequency (Core Clock)*/
/*----------------------------------------------------------------------------
  Clock functions
 *----------------------------------------------------------------------------*/
/**
 * Update SystemCoreClock variable
 *
 * @param  none
 * @return none
 *
 * @brief  Updates the SystemCoreClock with current core Clock
 *         retrieved from cpu registers.
 */
void SystemCoreClockUpdate (void)
{
    SystemCoreClock = __SYSTEM_CLOCK;
}
// -------------------------------------------------------------------------//
// SystemInit
// -------------------------------------------------------------------------//
void SystemInit(void)
{
    /* жλ */
    SCB->VTOR = (uint32_t)(&__Vectors);
    SystemCoreClock = __SYSTEM_CLOCK;
}

/********************************END OF FILE*********************************/
