/****************************************************************************
 * @file    : system.h
 * @author  : novosense
 * @version : V1.1.0
 * @date    : 03-05-2018
 * @brief   : System header File
 * @note
 * Copyright (C) 2019 novosense All rights reserved.
 ***************************************************************************/

// -------------------------------------------------------------------------//
// Define to prevent recursive inclusion
// -------------------------------------------------------------------------//
#ifndef __SYSTEM_H__
#define __SYSTEM_H__

#ifdef __cplusplus
extern "C" {
#endif

// -------------------------------------------------------------------------//
// Includes
// -------------------------------------------------------------------------//
#include "core_cm3.h"
#include "nsuc16x0.h"
#include "typedef.h"

// -------------------------------------------------------------------------//
// System Defines
// -------------------------------------------------------------------------//
#if (FPGA_EN > 0)
#define SYS_CLK 32000000
#else
#define SYS_CLK 32000000
#endif

#if (FPGA_EN > 0) && (DBG_PRINT_EN > 0)
#define DBG_Printf(format, ...) printf(format, ##__VA_ARGS__);
#elif (FPGA_EN > 0) && (DBG_RTT_EN > 0)
#define DBG_Printf(format, ...)                                                     \
    printf(format, ##__VA_ARGS__); // SEGGER_RTT_printf(0, format, ##__VA_ARGS__);
#else
#define DBG_Printf(format, ...)
#endif

// -------------------------------------------------------------------------//
// Private variables
// -------------------------------------------------------------------------//
void SystemInit(void);
void delay_us(u32 n);
void delay_ms(u32 n);
void PrintfInit(int baudRate);
void srand_set(u32 seed);
u32 rand_generate(uint32_t min, uint32_t max);
extern void (*const __vector_handlers[])(void); /*!<ж*/

void pattern_goto(u32 step);
void pattern_end(u32 flag);

void print_arrar_byte(uint8_t *abuf, uint32_t size, uint8_t align);
void print_arrar_halfword(uint8_t *abuf, uint32_t size, uint8_t align);
void print_arrar_word(uint8_t *abuf, uint32_t size, uint8_t align);
void print_register(uint32_t reg_addr, uint32_t size);

#ifdef __cplusplus
}
#endif

#endif
/********************************END OF FILE*********************************/
